import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _typeof from "@babel/runtime/helpers/esm/typeof";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _pick from 'lodash/pick';
import _assign from 'lodash/assign';
import _reduce from 'lodash/reduce';
import _forEach from 'lodash/forEach';
import _isEmpty from 'lodash/isEmpty';
import _keys from 'lodash/keys';
import _isEqual from 'lodash/isEqual';
import _mapValues from 'lodash/mapValues';
import { SET_INPUT } from './actionTypes';
import { getInputsFromState, inputsWithErrors } from '../util/helpers';
import { getReduxMountPoint } from '../util/mountPoint';
import log from '../util/log';
import { getDefaultInputs } from '../reducers';
var _promiseCache = {};

function _filterUnknownInputs(inputConfig, inputs) {
  // Only update known inputs
  return _reduce(inputs, function (result, value, key) {
    if (!inputConfig[key]) {
      log.error(key + ' is not a valid input.');
    } else {
      result[key] = value;
    }

    return result;
  }, {});
}
/**
 * Normalize an input state object removing unnecessary properties
 *
 * @private
 * @param input {Object}
 * @returns {Object}
 */


function _normalizeInput(input) {
  var normalized = _objectSpread({}, input); // Remove validating key if falsey


  if (!normalized.validating) {
    delete normalized.validating;
  }

  return normalized;
}
/**
 * Normalize all input states
 *
 * @private
 * @param inputs {Object}
 * @return {Object}
 */


function _normalizeInputs(inputs) {
  return _mapValues(inputs, _normalizeInput);
}

export function _setInputs(inputConfig, update) {
  var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return {
    type: SET_INPUT,
    payload: update,
    error: !!inputsWithErrors(update),
    meta: _objectSpread({
      reduxMountPoint: getReduxMountPoint(inputConfig)
    }, meta)
  };
}

function _fireChanges(inputConfig, update, inputsState, state, dispatch) {
  _forEach(update, function (input, key) {
    var _ref = inputConfig[key] || {},
        onChange = _ref.onChange;

    if (onChange) {
      onChange(input, inputsState, state, dispatch);
    }
  });
}
/**
 * Sets inputs' states without validation
 *
 * Also fires onChange callbacks for changed inputs
 *
 * `update` example:
 *  {
 *      email: {
 *          value: 'asdf@asdf.com'
 *      }
 *  }
 */


function _setInputs3(inputConfig, update) {
  var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return function (dispatch, getState) {
    var filteredUpdate = _filterUnknownInputs(inputConfig, update);

    var normalizedUpdate = _normalizeInputs(filteredUpdate);

    dispatch(_setInputs(inputConfig, normalizedUpdate, meta));

    if (!meta.suppressChange) {
      var state = getState();
      var inputsState = getInputsFromState(inputConfig, state);

      _fireChanges(inputConfig, normalizedUpdate, inputsState, state, dispatch);
    }

    return Promise.resolve(normalizedUpdate);
  };
}

export { _setInputs3 as setInputs };

function _resetInputs(inputConfig, inputKeys) {
  var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var update = getDefaultInputs(inputConfig);
  return _setInputs3(inputConfig, inputKeys ? _pick(update, inputKeys) : update, _objectSpread({
    reset: true
  }, meta));
}
/**
 * Validates the current state of inputs corresponding to the given keys
 *
 * @param inputConfig
 * @param inputKeys {Array}
 * @returns {Thunk}
 */


export { _resetInputs as resetInputs };

function _validateInputs(inputConfig, inputKeys) {
  var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return function (dispatch, getState) {
    var inputsState = getInputsFromState(inputConfig, getState());

    var keys = inputKeys || _keys(inputsState);

    var inputsToValidate = _reduce(keys, function (result, key) {
      var input = inputsState[key];

      if (inputConfig[key] && input) {
        result[key] = typeof input.error !== 'undefined' ? input.error : input.value;
      }

      return result;
    }, {});

    return _updateAndValidate(inputConfig, inputsToValidate, _objectSpread({}, meta, {
      validate: true
    }))(dispatch, getState);
  };
}
/**
 * Shortcut for setting just the values of the given inputs.
 *
 * @param inputsConfig
 * @param newValues {Object} key-value pair of new values
 * @param meta {Object}
 * @returns {Thunk}
 */


export { _validateInputs as validateInputs };

function _setValues(inputsConfig) {
  var newValues = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return function (dispatch) {
    var update = _mapValues(newValues, function (value) {
      return {
        value: value
      };
    });

    return dispatch(_setInputs3(inputsConfig, update, meta));
  };
}
/**
 * Add errors to the given inputKeys
 *
 * @param inputsConfig
 * @param inputKeys {Array}
 * @returns {Thunk}
 */


export { _setValues as setValues };

function _setErrors(inputsConfig) {
  var inputKeys = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return function (dispatch, getState) {
    var inputsState = getInputsFromState(inputsConfig, getState());

    var update = _reduce(inputKeys, function (result, key) {
      // Keep the existing value and async validating state, overwrite
      // everything else.
      var input = _pick(inputsState[key], ['value', 'validating']);

      result[key] = _objectSpread({}, input, {
        error: typeof inputsState[key].value === 'undefined' ? '' : inputsState[key].value
      });
      return result;
    }, {});

    return dispatch(_setInputs3(inputsConfig, update, meta));
  };
}

export { _setErrors as setErrors };

function _updateAndValidate(inputConfig, update) {
  var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return function (dispatch, getState) {
    var inputs = _filterUnknownInputs(inputConfig, update);

    if (_isEmpty(inputs)) {
      return Promise.resolve();
    }

    var state = getState();
    var inputsState = getInputsFromState(inputConfig, state);
    var promises = [];

    var createNewState = function createNewState(newInputState) {
      return _normalizeInput(meta.initialize ? _objectSpread({
        pristine: true
      }, newInputState) : newInputState);
    };

    var changes = _reduce(inputs, function (result, value, key) {
      var validator = inputConfig[key].validator;

      var _ref2 = inputsState[key] || {},
          prev = _ref2.value,
          currentlyValidating = _ref2.validating;

      var unchanged = _isEqual(prev, value);

      var cachedValidation = unchanged && currentlyValidating && _promiseCache[key];
      var validationResult = cachedValidation ? cachedValidation : validator ? validator(value, inputsState, state, dispatch) : true;
      var hasAsync = _typeof(validationResult) === 'object' && !!validationResult.then;

      var dispatchAndReturnPromiseResult = function dispatchAndReturnPromiseResult(inputState) {
        return _setInputs3(inputConfig, _defineProperty({}, key, createNewState(inputState)), meta)(dispatch, getState);
      };

      if (typeof validationResult === 'boolean' || typeof validationResult === 'string' || hasAsync || !validationResult) {
        // Or Promise
        var change = validationResult === true || hasAsync ? {
          // True or hasAsync, set value
          value: value,
          validating: currentlyValidating || hasAsync && !unchanged // Will be validating if async validator exists

        } : {
          // False returned, input invalid
          value: prev,
          error: typeof value === 'undefined' ? '' : value,
          validating: false
        }; // Add errorText if validator returned a string

        if (typeof validationResult === 'string') {
          change.errorText = validationResult;
        }

        var newState = createNewState(change); // Only fire setInputs on state that is different than current.
        // If the only difference in state is that pristine is missing from
        // the new state, then we should still consider this pristine.

        if (!_isEqual(newState, inputsState[key]) && !_isEqual(_objectSpread({}, newState, {
          pristine: true
        }), inputsState[key])) {
          result[key] = newState;
        }

        if (!change.validating) {
          promises.push(Promise.resolve(_defineProperty({}, key, newState)));
        } else {
          var resultPromise = cachedValidation ? cachedValidation : _promiseCache[key] = validationResult.then( // Passed validation
          function () {
            return dispatchAndReturnPromiseResult({
              value: value
            });
          }, // Failed validation
          function (errorText) {
            return dispatchAndReturnPromiseResult(_objectSpread({
              value: prev,
              error: typeof value === 'undefined' ? '' : value
            }, typeof errorText === 'string' ? {
              errorText: errorText
            } : {}));
          }); // Kick off async

          promises.push(resultPromise);
        }
      } else {
        log.error("\n                Value returned from validator must be a\n                boolean representing valid/invalid, a string representing errorText, or a promise for performing async\n                validation. Got ".concat(_typeof(validationResult), " instead.\n            "));
      }

      return result;
    }, {});

    if (!_isEmpty(changes)) {
      _setInputs3(inputConfig, changes, meta)(dispatch, getState);
    }

    return new Promise(function (resolve, reject) {
      Promise.all(promises).then(function (results) {
        var resultObject = _assign.apply(void 0, [{}].concat(_toConsumableArray(results)));

        var erroredInputs = inputsWithErrors(resultObject);

        if (erroredInputs) {
          reject(erroredInputs);
        } else {
          resolve(resultObject);
        }
      });
    });
  };
}

export { _updateAndValidate as updateAndValidate };

function _initializeInputs(inputconfig, update) {
  var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return _updateAndValidate(inputconfig, update, _objectSpread({
    initialize: true
  }, meta));
}

export { _initializeInputs as initializeInputs };
export var bindActions = function bindActions(inputConfig) {
  return {
    setInputs: function setInputs(update, meta) {
      return _setInputs3(inputConfig, update, meta);
    },
    updateAndValidate: function updateAndValidate(update, meta) {
      return _updateAndValidate(inputConfig, update, meta);
    },
    validateInputs: function validateInputs(inputKeys, meta) {
      return _validateInputs(inputConfig, inputKeys, meta);
    },
    setValues: function setValues(newValues, meta) {
      return _setValues(inputConfig, newValues, meta);
    },
    setErrors: function setErrors(inputKeys, meta) {
      return _setErrors(inputConfig, inputKeys, meta);
    },
    resetInputs: function resetInputs(inputKeys, meta) {
      return _resetInputs(inputConfig, inputKeys, meta);
    },
    initializeInputs: function initializeInputs(update, meta) {
      return _initializeInputs(inputConfig, update, meta);
    }
  };
};